// CleanUpHTMLDoc.cpp : implementation of the CCleanUpHTMLDoc class
//
#include "stdafx.h"
#include "MyIncludes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCleanUpHTMLDoc

IMPLEMENT_DYNCREATE(CCleanUpHTMLDoc, CDocument)

BEGIN_MESSAGE_MAP(CCleanUpHTMLDoc, CDocument)
	//{{AFX_MSG_MAP(CCleanUpHTMLDoc)
	ON_COMMAND(ID_MY_FILE_OPEN, OnMyFileOpen)
	ON_COMMAND(ID_MY_FILE_SAVE_AS, OnMyFileSaveAs)
	ON_COMMAND(ID_MY_FILE_SAVE, OnMyFileSave)
	ON_COMMAND(ID_MY_FILE_NEW, OnMyFileNew)
	ON_COMMAND(ID_MY_FILE_CLOSE, OnMyFileNew)
	ON_COMMAND(ID_SHOW_IN_EXPLORER, OnShowInExplorer)
	ON_UPDATE_COMMAND_UI(ID_SHOW_IN_EXPLORER, OnUpdateShowInExplorer)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCleanUpHTMLDoc construction/destruction

CCleanUpHTMLDoc::CCleanUpHTMLDoc()
{
	m_lastOpenedFileStr = ("");
	m_bFirstTime = TRUE;

	CCleanUpHTMLApp * pApp = (CCleanUpHTMLApp *)AfxGetApp();
	// get settings from registry
	m_openInitDirStr = pApp->GetProfileString(settings, openDirStr, ".\\");
	m_saveAsInitDirStr = pApp->GetProfileString(settings, saveDirStr, ".\\");
	m_saveFilterIndex = 1; // one-based
	m_openFilterIndex= 1; // one-based
	m_bInhibitModifyChange = FALSE;
	m_numTimesSaved = 0;
}

CCleanUpHTMLDoc::~CCleanUpHTMLDoc()
{
	CCleanUpHTMLApp * pApp = (CCleanUpHTMLApp *)AfxGetApp();
	// save settings in registry
	pApp->WriteProfileString(settings, openDirStr, m_openInitDirStr);
	pApp->WriteProfileString(settings, saveDirStr, m_saveAsInitDirStr);
}

BOOL CCleanUpHTMLDoc::OnNewDocument()
{
	const char procStr[] = "DicksEditor FIle Close";

	if(m_bFirstTime) // on startup there is no view yet created
	{
		if(!CDocument::OnNewDocument()) return FALSE;
		((CEditView*)m_viewList.GetHead())->SetWindowText(NULL);
		m_bFirstTime = FALSE;
		return TRUE;
	}

	if(IsModified())
	{
		UINT retVal = FALSE;

		if(IDYES == (retVal = ((CMainFrame *)AfxGetMainWnd())->MessageBox(
		   "This Document has been modified!\nSave contents first?", 
		   procStr, MB_YESNOCANCEL | MB_ICONQUESTION)))
		{
			OnMyFileSaveAs();
		}
		else if(retVal == IDCANCEL)
		{
			return FALSE;
		}
	}

	if(!CDocument::OnNewDocument()) return FALSE;
	((CEditView*)m_viewList.GetHead())->SetWindowText(NULL);

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	CCleanUpHTMLView * pEditView = (CCleanUpHTMLView *)pMainFrame->GetActiveView();
	ASSERT_VALID(pEditView);
	// all new documents can be written
	if(pEditView) pEditView->SetReadOnly(FALSE);

	return TRUE;
}

void CCleanUpHTMLDoc::OnMyFileNew() 
{
	if(OnNewDocument() == FALSE)
	{
		return;	
	}

	SetTitle("Untitled")
		;
	CCleanUpHTMLView * pEditView = (CCleanUpHTMLView*)m_viewList.GetHead();
	ASSERT_VALID(pEditView);
	if(pEditView)
	{
		pEditView->m_bIsUnix = FALSE;
		pEditView->OnInitialUpdate(); // gets document length etc.
	}
}

/////////////////////////////////////////////////////////////////////////////
// CCleanUpHTMLDoc serialization

void CCleanUpHTMLDoc::Serialize(CArchive& ar)
{
//	TRACE2("LOADING %d  STORING %d\n", ar.IsLoading(), ar.IsStoring());
	// CEditView contains an edit control which handles all serialization
	((CCleanUpHTMLView *)m_viewList.GetHead())->MySerializeRaw(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CCleanUpHTMLDoc diagnostics

#ifdef _DEBUG
void CCleanUpHTMLDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCleanUpHTMLDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCleanUpHTMLDoc commands

void CCleanUpHTMLDoc::SetModifiedFlag(BOOL bModified)
{
	if(!m_bInhibitModifyChange) // dont change '*' in title while sending mail
	{
		if(bModified && !IsModified())
		{
			SetTitle(GetTitle() + strIndicator);
		}

		if(IsModified() && !bModified) // remove "*" to view's title bar
		{
			CString strTitle = GetTitle();
			SetTitle(strTitle.Left(strTitle.GetLength() - STR_INDICATOR_LEN));
		}
	}

	UpdateFrameCounts();
	CDocument::SetModifiedFlag(bModified);
}

const char * const myOpenFilter = "Internet Files ( *.html *.htm *.css *.pl *.js *.php )|*.html;*.htm;*.css;*.pl;*.js;*.php|"
								  "Text Files ( *.txt; *.asc )|*.txt;*.asc|"
								  "Comma Delimited Files ( *.csv; *.prn )|*.csv;*.prn|"
								  "All Files ( *.* )|*.*||";

const char * const dfltSaveExt[] = {".html", ".txt", ".csv", ""};

const UINT myOpenFlags   = OFN_EXPLORER | OFN_ENABLESIZING | OFN_PATHMUSTEXIST;
const UINT mySaveAsFlags = OFN_EXPLORER | OFN_ENABLESIZING | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;

const char const myOpenFileTitle[]    = "DicksEditor File Open";
const char const mySaveAsFileTitle[]  = "DicksEditor File Save As";
const char const mySaveFileTitle[]    = "DicksEditor File Save";
const char const myCompareFileTitle[] = "File to Compare with DickEditor";

CString CCleanUpHTMLDoc::GetFilePathName()
{
	CFileDialog fDlg(TRUE, NULL, m_lastOpenedFileStr, myOpenFlags | OFN_HIDEREADONLY, myOpenFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_openInitDirStr;
	fDlg.m_ofn.lpstrTitle = myOpenFileTitle;

	CCleanUpHTMLView * pEditView = (CCleanUpHTMLView*)m_viewList.GetHead();
	ASSERT_VALID(pEditView);
	// assumes read only flag was not set in myOpenFlags
	fDlg.m_ofn.Flags |= (pEditView->GetReadOnly() ? OFN_READONLY : 0);
	fDlg.m_ofn.nFilterIndex = m_openFilterIndex;
	fDlg.m_ofn.lpstrTitle = myCompareFileTitle;

	if(IDOK == fDlg.DoModal())
	{
		CString filePath = fDlg.GetPathName();
		m_lastOpenedFileStr = fDlg.GetFileName();
		m_openInitDirStr = filePath.Left(filePath.GetLength() - m_lastOpenedFileStr.GetLength());
		m_openFilterIndex = fDlg.m_ofn.nFilterIndex;
		return filePath;
	}

	return CString("");
}

void CCleanUpHTMLDoc::OnMyFileOpen() 
{
	CFileDialog fDlg(TRUE, NULL, m_lastOpenedFileStr, myOpenFlags, myOpenFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_openInitDirStr;
	fDlg.m_ofn.lpstrTitle = myOpenFileTitle;

	CCleanUpHTMLView * pEditView = (CCleanUpHTMLView*)m_viewList.GetHead();
	ASSERT_VALID(pEditView);
	// assumes read only flag was not set in myOpenFlags
	fDlg.m_ofn.Flags |= (pEditView->GetReadOnly() ? OFN_READONLY : 0);
	fDlg.m_ofn.nFilterIndex = m_openFilterIndex;

	if(IDOK == fDlg.DoModal())
	{
		CString filePath = fDlg.GetPathName();
		m_lastOpenedFileStr = fDlg.GetFileName();
		m_openInitDirStr = filePath.Left(filePath.GetLength() - m_lastOpenedFileStr.GetLength());
		pEditView->SetReadOnly(fDlg.GetReadOnlyPref());
		m_openFilterIndex = fDlg.m_ofn.nFilterIndex;

		if(IsModified()) // ask to save before opening a new file
		{
			CString pathName = GetPathName();
			CString fileName = pathName.Mid(pathName.ReverseFind('\\') + 1);
			CString aStr;
			aStr.Format("%shas been modified!\nSave contents first?", fileName.IsEmpty() ? "This Document " : fileName + "\n");
			UINT retVal = 0;

			if(IDYES == (retVal = ((CMainFrame *)AfxGetMainWnd())->MessageBox(aStr, myOpenFileTitle, MB_YESNOCANCEL | MB_ICONQUESTION)))
			{
				if(pathName.IsEmpty())
				{
					OnMyFileSaveAs();
				}
				else
				{
					OnSaveDocument(pathName); // saves modified when another file opens 
				}
			}
			else if(retVal == IDCANCEL) // cancel both save and open
			{
				return;
			}
		}

		OnOpenDocument(filePath);
		SetPathName(filePath, TRUE); // add to MRU list
		pEditView->OnInitialUpdate(); // gets document length etc.
		m_numTimesSaved = 0;
	}
}

void CCleanUpHTMLDoc::OnMyFileSaveAs() 
{
	CString pathName = GetPathName();
	CString fileName = pathName.Mid(pathName.ReverseFind('\\') + 1);

	CFileDialog fDlg(FALSE, NULL, fileName, mySaveAsFlags, myOpenFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_saveAsInitDirStr;
	fDlg.m_ofn.lpstrTitle = mySaveAsFileTitle;
	fDlg.m_ofn.nFilterIndex = m_saveFilterIndex;

	if(IDOK == fDlg.DoModal())
	{
		CString fileName = fDlg.GetFileName();
		CString filePath = fDlg.GetPathName();
		m_saveFilterIndex = fDlg.m_ofn.nFilterIndex; // one-based

		if(fileName.ReverseFind('.') < 0) // extension ?
		{
			// add default extension
			fileName += dfltSaveExt[m_saveFilterIndex - 1]; // index is 1-based
			filePath += dfltSaveExt[m_saveFilterIndex - 1];
		}

		m_saveAsInitDirStr = filePath.Left(filePath.GetLength() - fileName.GetLength());

		OnSaveDocument(filePath); // saves with a new name
		SetPathName(filePath, TRUE);
		m_numTimesSaved++;
	}
}

void CCleanUpHTMLDoc::OnMyFileSave() 
{
	CString filePath = GetPathName();
	CString fileName = filePath.Mid(filePath.ReverseFind('\\') + 1);

	if(fileName.IsEmpty()) // an untitled new file
	{
		CFileDialog fDlg(FALSE, NULL, "untitled", mySaveAsFlags, myOpenFilter, NULL);
			
		fDlg.m_ofn.lpstrInitialDir = m_saveAsInitDirStr;
		fDlg.m_ofn.lpstrTitle = mySaveFileTitle;

		if(IDOK == fDlg.DoModal())
		{
			filePath = fDlg.GetPathName();
			fileName = fDlg.GetFileName();
			m_saveAsInitDirStr = filePath.Left(filePath.GetLength() - fileName.GetLength());
			OnSaveDocument(filePath); // saves untitled with new name
			SetPathName(filePath, TRUE);
			m_numTimesSaved++;
		}
	}
	else
	{
		OnSaveDocument(filePath); // normal save runs silent
		SetPathName(filePath, TRUE);
		m_numTimesSaved++;
	}
}

BOOL CCleanUpHTMLDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	m_numTimesSaved = 0;

	if(!CDocument::OnOpenDocument(lpszPathName)) return FALSE;
	return TRUE;
}

BOOL CCleanUpHTMLDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	return CDocument::OnSaveDocument(lpszPathName);
}

void CCleanUpHTMLDoc::OnFileSendMail() 
{
	m_bInhibitModifyChange = TRUE;
	BOOL bModified = IsModified();
	SetModifiedFlag(TRUE);
	
	CDocument::OnFileSendMail();
	
	SetModifiedFlag(bModified);
	m_bInhibitModifyChange = FALSE;
}

void CCleanUpHTMLDoc::OnUpdateFileSendMail(CCmdUI* pCmdUI)
{
	CCleanUpHTMLView * pEditView = (CCleanUpHTMLView*)m_viewList.GetHead();
	ASSERT_VALID(pEditView);

	if(pEditView->GetBufferLength())
	{
		CDocument::OnUpdateFileSendMail(pCmdUI);
	}
	else
	{
		pCmdUI->Enable(FALSE);
	}
}

CString CCleanUpHTMLDoc::GetSaveAsDirStr()
{
	return m_saveAsInitDirStr;
}

void CCleanUpHTMLDoc::OnShowInExplorer() 
{
	static char procName[] = _T("Show in Explorer");

	// hilight directory and file in Windows Explorer
	CString runStr(_T("explorer.exe /e,/select,"));
	CString pathStr = GetPathName();
	runStr += pathStr;

	if(::WinExec(runStr, SW_SHOW) <= 31)
	{
		AfxGetMainWnd()->MessageBox(GetLastErrorMessage(), procName, MB_ICONERROR);
	}
	else
	{
		PutStrOnClipBoard(pathStr);
	}
}

void CCleanUpHTMLDoc::OnUpdateShowInExplorer(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!GetPathName().IsEmpty());	
}

void CCleanUpHTMLDoc::DeleteContents() 
{
	// TODO: Add your specialized code here and/or call the base class
	if(m_viewList.GetHeadPosition())
	{
		((CCleanUpHTMLView*)m_viewList.GetHead())->UpdateFileStatus(FALSE); // save status
	}

	CDocument::DeleteContents();
}
