#include "stdafx.h"
#include "MyIncludes.h"

BOOL EkCreateProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage,
							int nPaneIndex,	// if -1 use whole status bar width
							int cxMargin,
							int cxMaxWidth,
							UINT nIDControl)
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
	int pIndx = nPaneIndex < 0 ? 0 : nPaneIndex;

    CRect rc;
    pStatusBar->GetItemRect(pIndx, &rc );

	if(nPaneIndex < 0)
	{
		CRect statusBarRect;
		pStatusBar->GetWindowRect(&statusBarRect);
		rc.right = rc.left + statusBarRect.Width();
	}

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText(pIndx, szMessage );
	pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(PBS_SMOOTH | WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

int FileSize( const char * szFileName ) 
{ 
  struct stat fileStat; 
  int err = stat( szFileName, &fileStat ); 
  if (0 != err) return 0; 
  return fileStat.st_size; 
}

BOOL GetStrOnClipBoard(CString & aStr)
{
	if(::OpenClipboard(NULL))
	{
		HANDLE hData = ::GetClipboardData(CF_TEXT);
		
		if(hData)
		{
			ULONG nSize = GlobalSize(hData);

			if(nSize)
			{
				LPTSTR pBuf = aStr.GetBufferSetLength(nSize);
				LPSTR pData = (LPSTR)::GlobalLock(hData);

				::lstrcpy(pBuf, pData);

				aStr.ReleaseBuffer(-1);
				::GlobalUnlock(hData);
				::CloseClipboard();

				return TRUE;
			}
		}
	}

	return FALSE;
}

BOOL PutStrOnClipBoard(CString & aStr)	// overload on following proceedure using char *
{
	char * cBuf = aStr.GetBuffer(aStr.GetLength() + 1);

	BOOL bRet = PutStrOnClipBoard(cBuf);

	aStr.ReleaseBuffer(-1);

	return bRet;
}

BOOL PutStrOnClipBoard(char * cBuf)
{
	if(::OpenClipboard(NULL))	// copy to clipboard as aa text string
	{
		UINT len = ::strlen(cBuf) + 1;

		::EmptyClipboard();
		HANDLE hData = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, len);
		LPSTR pData = (LPSTR)::GlobalLock(hData);

		::CopyMemory(pData, cBuf, len);
		::GlobalUnlock(hData);
		::SetClipboardData(CF_TEXT, hData);
		::CloseClipboard();
		return TRUE;
	}

	return FALSE;
}

CString AddCRtoLF(CString & aStr)	// add CR to LF
{
	CString bStr(_T(""));

	int len = aStr.GetLength();

	for(int i = 0; i < len; i++)
	{
		TCHAR c = aStr[i];

		if(c != _T('\r'))
		{
			if(c == _T('\n'))
			{
				bStr += _T("\r\n");
			}
			else
			{
				bStr += c;
			}
		}
	}

	return bStr;
}

void EkChangePrintingOrientation( CDC* pDC, CPrintInfo* pInfo, short NewOrientation )
{
	ASSERT_VALID( pDC );
	ASSERT( pInfo != NULL );

	DEVMODE* pDevMode = pInfo->m_pPD->GetDevMode();

	// Only change if new orientation is different 
	if( pDevMode->dmOrientation != NewOrientation )
	{
		pDevMode->dmOrientation = NewOrientation;
		pDC->ResetDC( pDevMode );
	}
}

void EkChangePrintingPaperSize( CDC* pDC, CPrintInfo* pInfo, short NewPaperSize )
{
	ASSERT_VALID( pDC );
	ASSERT( pInfo != NULL );

	DEVMODE* pDevMode = pInfo->m_pPD->GetDevMode();

	// Only change if new orientation is different 
	if( pDevMode->dmPaperSize != NewPaperSize )
	{
		pDevMode->dmPaperSize = NewPaperSize;
		pDC->ResetDC( pDevMode );
	}
}

#include <Wininet.h>
CString GetLastErrorMessage()
{
	DWORD nErr = GetLastError();
	CString msgStr;

	if(nErr == ERROR_INTERNET_EXTENDED_ERROR)
	{
		DWORD dwError;
		char szBuffer[1024];
		DWORD dwBufferLength = sizeof(szBuffer);

		if(InternetGetLastResponseInfo(&dwError, szBuffer, &dwBufferLength))
		{
			 msgStr = szBuffer;
		}
	}
	else
	{
		LPVOID lpMsgBuf;

		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			nErr,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);

		msgStr = (LPCTSTR)lpMsgBuf;
		LocalFree(lpMsgBuf);
	}

	msgStr.TrimRight(); // remove CR-LF
	msgStr.TrimLeft();

	if(msgStr.IsEmpty())
	{
		msgStr.Format(_T("Last Error = %d"), nErr);
	}

	return msgStr;
}

int CALLBACK BrowseForFolderCallback(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData)
{
	char szPath[MAX_PATH];

	switch(uMsg)
	{
		case BFFM_INITIALIZED:
			SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
			break;

		case BFFM_SELCHANGED: 
			if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szPath)) 
			{
				SendMessage(hwnd, BFFM_SETSTATUSTEXT,0,(LPARAM)szPath);	

			}
			break;
	}

	return 0;
}

BOOL BrowseFolders(HWND hwnd, LPSTR lpszFolder, LPSTR lpszTitle)
{
	BROWSEINFO bi;
	char szPath[MAX_PATH + 1];
	LPITEMIDLIST pidl;
	BOOL bResult = FALSE;

	LPMALLOC pMalloc;

    if (SUCCEEDED(SHGetMalloc(&pMalloc))) 
	{
		bi.hwndOwner = hwnd;
		bi.pidlRoot = NULL;
		bi.pszDisplayName = NULL;
		bi.lpszTitle = lpszTitle;
		bi.ulFlags = BIF_RETURNONLYFSDIRS; //BIF_EDITBOX | BIF_STATUSTEXT
		bi.lpfn = BrowseForFolderCallback;
		bi.lParam = (LPARAM)lpszFolder;
		
		pidl = SHBrowseForFolder(&bi);
		if (pidl)
		{
			if (SHGetPathFromIDList(pidl,szPath)) 
			{
				bResult = TRUE;
				strcpy(lpszFolder, szPath);
			}

		   pMalloc->Free(pidl);
		   pMalloc->Release();
            			
		}
	}

	return bResult;	
}

char * stristr(const char *string, const char * strCharSet )
{
	// verify parameters
	if(string == NULL || strCharSet == NULL )
	{
		return NULL;
	}

	// get length of the substring
	int nCharSetLen = strlen(strCharSet);

	// empty substring-return input (consistent with strstr)
	if(nCharSetLen == 0)
	{
		return (char *)string;
	}

	int nMaxOffset = strlen(string) - nCharSetLen;

	if(nMaxOffset < 0)
	{
		return NULL;
	}

	const char * pPos = string;

	for(int nOffset = 0; nOffset <= nMaxOffset; nOffset++)
	{
		if(strnicmp(pPos, strCharSet, nCharSetLen) == 0)
		{
			return (char *)pPos;
		}

		pPos++;
   }

   return NULL;
}

// file string may cointain whild cards
int FindInFolder(CString & folderStr, CString & fileStr, CString & findWhatStr, 
				  CString & resultStr, CString & msgStr, BOOL bMatchCase, BOOL bMatchWhole)
{
	WIN32_FIND_DATA fd;
	HANDLE h = NULL;
	UINT nFound = 0;

	if(INVALID_HANDLE_VALUE != (h = ::FindFirstFile(folderStr + fileStr, &fd)))
	{ 
		UINT bufferSize = 0;

		int nFindWhatLen = findWhatStr.GetLength();

		char * pBuf = NULL,
			 * pFindStr = findWhatStr.LockBuffer(),
			    // pointer to case sensitive or insensitive function
			 * (*pStrStr)(const char*, const char*) = bMatchCase ? &strstr : &stristr;

		CFile aFile;

		resultStr.Format(_T("\r\n%s\r\n"), folderStr);

		CString aStr, bStr;

		do
		{
			if((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0) // not a directory
			{
				if(fd.nFileSizeLow + 1 > bufferSize)	// add room for null terminator
				{
					delete []pBuf;
					pBuf = new char[fd.nFileSizeLow + 1];
					bufferSize = fd.nFileSizeLow + 1;

					if(!pBuf)
					{
						msgStr = GetLastErrorMessage();
						break;
					}
				}
				
				if(aFile.Open(folderStr + fd.cFileName, CFile::modeRead))
				{
					if(fd.nFileSizeLow == aFile.Read(pBuf, fd.nFileSizeLow)) // read all ? 
					{
						pBuf[fd.nFileSizeLow] = _T('\0'); // make it a null terminated string

						char * pOffset = NULL,
						     * pEnd = pBuf + fd.nFileSizeLow; // point to null terminator

						aStr.Empty();
						bStr.Empty();

						if(NULL != (pOffset = pStrStr(pBuf, pFindStr))) 
						{
							do
							{
								bStr.Format(_T("%u "), pOffset - pBuf);
								
								if(bMatchWhole) // check if whole word
								{
									if((pOffset == pBuf || !__iscsym(pOffset[- 1])) &&
									   (pOffset + nFindWhatLen == pEnd || !__iscsym(pOffset[nFindWhatLen])))
									{
										aStr += bStr;
									}
								}
								else
								{
									aStr += bStr;
								}
							}
							while(NULL != (pOffset = ::strstr(pOffset + 1, pFindStr)));
						}

						if(!aStr.IsEmpty()) 
						{
							resultStr = resultStr + _T("  ") + fd.cFileName + _T("  --  ") + aStr + _T("\r\n");
							nFound++;
						}

						aFile.Close();
					}
					else
					{
						msgStr = GetLastErrorMessage();
						break;
					}
				}
				else
				{
					msgStr = GetLastErrorMessage();
					break;
				}
			}
		}
		while(FindNextFile(h, &fd));

		FindClose(h);
		delete []pBuf;
		findWhatStr.UnlockBuffer();

		if(nFound == 0 || !msgStr.IsEmpty())
		{
			resultStr.Empty();
		}
	}
	else // no files in this folder
	{
		msgStr.Empty();
		resultStr.Empty();
	}

	return nFound;
}
